confOmega <- function(y, al, m, alpha = 0.1, lambda, lambdaS, qA){
  if(missing(y)){
    stop("Data must be specified.")
  }
  n <- length(y)
  if(missing(al)){
    stop("The alphabet must be specified.")
  }
  if( length(al) < 2){
    stop("Alphabet must contain at least two values.")
  }
  al<-sort(al)
  if(missing(m)){
    stop("Number of source functions must be specified.")
  }
  if(m < 2){
    stop("There must be more then one source function.")
  }
  if(missing(lambda)){
    sigma <- stepR::sdrobnorm(y)
    lam <- seq(0,1,0.00001)
    lambda <- max(lam[which(lam / log(1/lam) >= 2 * (sigma * m / (al[2] - al[1]))^2 / n)[1]], 1/n)
  }
  if(missing(lambdaS)){
    lambdaS <- lambda
  } 
  
  
  minIl <- lambdaS * n
  minIg <- lambda * n
  digits <- 2
  tsh <- (al[2] - al[1]) * 10^( -digits )
  am <- expand.grid(rep(list(al), m))
  amI <- t(apply(am, 1, function(x) order(x))) - 1
  
  
  if(missing(qA)){
    boundsR <- boundsGen(y = y, alpha = alpha)
  }else{
    boundsR <- boundsGen(y = y, qA = qA)
  }
  
  lengB <- unique(boundsR$ri - boundsR$li )
  
  minIl <- max(lengB[which(lengB <= minIl)])
  minIg <- max(lengB[which(lengB <= minIg)])
  
  #minimal overall interval length  
  remove <- which( boundsR$ri - boundsR$li < minIg)
  if(length(remove) > 0){
    bouR <- boundsR[-remove,]
  }else{
    bouR <- boundsR
  }
  
  #minimal local interval length
  remove <- which(boundsR$ri - boundsR$li < minIl)
  if(length(remove) > 0){
    bou <- unique( round( boundsR[-remove, 3 : 4], digits = -log(tsh)/log(10) ), MARGIN = 1)
  }else{
    bou <- unique( round( boundsR[ ,3:4], digits = -log(tsh)/log(10) ), MARGIN = 1)
  }
  
  if(dim(bou)[1] == 0){
    ci <- round(data.frame( lower = c(numeric(m-1), 1/m), upper = 1 / (m - 0 : (m-1)) ), digits = 2)
    attr(ci, "uninfo") <- TRUE
    return(ci)
  } 

  # alphabet correction for bounds
  
  bouR[,3] <- pmax(bouR[,3], al[1])
  bouR[,4] <- pmax(bouR[,4], al[1])
  bouR[,3] <- pmin(bouR[,3], max(al))
  bouR[,4] <- pmin(bouR[,4], max(al))
  
  bou[,1] <- pmax(bou[,1], al[1])
  bou[,2] <- pmax(bou[,2], al[1])
  bou[,1] <- pmin(bou[,1], max(al))
  bou[,2] <- pmin(bou[,2], max(al))
  
    
  indNb <- min(bouR$li) : max(bouR$ri)
  for(i in 1:dim(bouR)[1]){
    indNb <- setdiff( indNb, bouR$li[i] : bouR$ri[i])
  }
  if( length( indNb ) > 0 ){
    ci <- round(data.frame( lower = c(numeric(m-1), 1/m), upper = 1 / (m - 0 : (m-1)) ), digits = 2)
    attr(ci, "uninfo") <- TRUE
    return(ci)  }
  
  # Determine Confidence Box
  
  ##### Omega 1
  index <- which( bou[ ,2] > al[1] & bou[ ,1] < al[1] + (al[2] - al[1]) / m)
  if( length( index ) == 0 ){
    ci <- round(data.frame( lower = c(numeric(m-1), 1/m), upper = 1 / (m - 0 : (m-1)) ), digits = 2)
    attr(ci, "uninfo") <- TRUE
    return(ci)  }
  
  idVal <- cbind( pmax( bou[ index, 1], al[1] ), pmin( bou[ index, 2], ( al[1] + (al[2] - al[1]) / m)))
  idVal <- matrix( unique( round( idVal, digits = -log(tsh) / log(10)), MARGIN = 1), ncol = 2)
  idVal <- maxEl( idVal )
  
  omega <- matrix( 0, nrow = max( 1, dim(idVal)[1] ), ncol = 2 * m)
  omega[, 1:  2] <- as.matrix( omegaCal( idVal, al = al ) )
  if(m > 2){
    omega[, 3 : 4] <- cbind( omega[ ,1] , (1 - omega[ ,1]) / (m-1) )
    if( dim( idVal )[1] > 1 ){
      if(m > 3){ 
        for(i in 3 : (m-1) ){
          omega[ , (2 * i - 1) : (2 * i) ] <- cbind( omega[ ,2 * (i - 1) - 1] , ( 1 - rowSums( omega[ ,seq( from = 1, to = 2 * (i - 1) - 1, by = 2 ) ] ) ) / ( m - i + 1 ) )
        }
      }
      omega[, (2 * m - 1) : (2 * m)] <- cbind( pmax(1 - rowSums( omega[ , seq( from = 2, to = 2 * (m - 1) , by = 2 ) ] ), omega[, (2 * m - 3) ], 1/m  ) , 1 - rowSums( omega[ ,seq( from = 1, to = 2 * (m - 1) - 1, by = 2 )] ) )
    }else{
      if(m > 3){ 
        for(i in 3 : (m - 1)){
          omega[ ,(2 * i - 1) : (2 * i)] <- cbind( omega[ ,2 * (i - 1) - 1], (1 - sum( omega[ ,seq( from = 1, to = 2 * (i - 1) - 1, by = 2)] ) ) / (m - i + 1) )
        }
      }
      omega[ , (2 * m - 1) : (2 * m)] <- cbind( pmax(1 - sum( omega[ ,seq( from = 2, to = 2 * (m - 1), by = 2)] ),  omega[ , (2 * m - 3)], 1/m ), 1 - sum( omega[ ,seq( from = 1, to = 2 * (m - 1) - 1, by = 2 )] ) )
      
    }
  }else{
    omega[, 3 : 4] <- cbind( 1 - (omega[ ,seq( from = 2, to = 2 * (m - 1), by = 2 )] ), 1 - (omega[ ,seq( from = 1, to = 2 * (m - 1) - 1, by = 2)] ) )
  }
  
  omega <- round(omega, digits)

  remove <- explainVal( omega, bou = bouR, am = am, amI = amI, indNb = indNb)
  
  if( length( remove ) != 0){
    idVal <- matrix( idVal[ -remove, ], ncol = 2)
  }
  
  if(length( idVal ) == 0){
    ci <- round(data.frame( lower = c(numeric(m-1), 1/m), upper = 1 / (m - 0 : (m-1)) ), digits = 2)
    attr(ci, "uninfo") <- TRUE
    return(ci)
  }
  
  
  #### Omega 2 - Omega m-1  
  if(m > 2){
    for(i in 1 : (m - 2) ){
      idVal <- matrix(idVal, ncol = 2 * i)
      nex <- corBox( idVal, bou, al = al, m = m)
      lb <- sapply( nex, function(x) length( as.numeric( x )) / 2)
      if(max(lb) == 0){
        ci <- round(data.frame( lower = c(numeric(m-1), 1/m), upper = 1 / (m - 0 : (m-1)) ), digits = 2)
        attr(ci, "uninfo") <- TRUE
        return(ci)
      }
      
      idVal <- matrix( apply( idVal, 2, function(x) {rep( x, lb )} ), ncol = 2 * i)
      nex <- matrix( do.call( "rbind", nex), ncol = 2 )

      if(i == 1){
        idVal[, 2] <- pmin(idVal[, 2], nex[ ,2], al[2] + (m-1)*al[1] - (m - 1) * nex[,1] )
      }
      if(i == 2){
        idVal[, 4] <- pmin(idVal[, 4], nex[ ,2], al[2] + (m-1)*al[1]  - idVal[,1] - (m - 2) * nex[,1] )
        idVal[, 2] <- pmin(idVal[,2], al[2] + (m-1)*al[1] - idVal[, 3] - (m - 2) * nex[,1])
      }
      if(i > 2){
        if(dim(idVal)[1] > 1){
          idVal[, 2 * i] <- pmin(idVal[, 2 * i], nex[ ,2], 
                                 al[2] + (m-1)*al[1]  - rowSums(idVal[ , seq(1, 2*i - 2, 2)]) - (m - i) * nex[,1] )
          idVal[, seq(2, 2*i -2, 2)] <- sapply(seq(2, 2*i -2, 2), 
                                               function(x) pmin(idVal[,x], al[2] + (m-1)*al[1] - rowSums(idVal[, seq(1, 2 * i, 2)]) + idVal[,x-1] - (m - i) * nex[,1]) ) 
          
        }else{
          idVal[, 2 * i] <- pmin(idVal[, 2 * i], nex[ ,2], 
                                 al[2] + (m-1)*al[1]  - sum(idVal[ , seq(1, 2*i - 2, 2)]) - (m - i) * nex[,1] )
          idVal[, seq(2, 2*i -2, 2)] <- sapply(seq(2, 2*i -2, 2), 
                                               function(x) pmin(idVal[,x], al[2] + (m-1)*al[1] - sum(idVal[, seq(1, 2 * i, 2)]) + idVal[,x-1] - (m - i) * nex[,1]) ) 
          
        }
           }
       
      idVal <- cbind( idVal, nex )
      idVal <- matrix( unique( round( idVal, digits = -log(tsh) / log(10) ), MARGIN = 1), ncol = 2 * i + 2)
      omega <- matrix( 0, nrow = max( 1, dim( idVal )[1]), ncol = 2 * m )
      omega[ , 1:  dim(idVal)[2]] <- as.matrix( omegaCal( val = idVal, al = al))
      if( dim(idVal)[1] > 1 ){
        if(( i + 2 ) < m){
          for(k in (i + 2) : (m - 1)){
            omega[,( 2 * k - 1) : (2 * k)] <- cbind( omega[, 2 * (k - 1) - 1], ( 1 - rowSums( omega[ , seq( 1 , 2 * ( k - 1 ) - 1, 2)])) / ( m - k + 1))
          }
        }
        if(2 < m){
          omega[ , (2 * m - 1) : (2 * m)] <- cbind( pmax( 1 - rowSums( omega[ , seq( from = 2, to = 2 * ( m - 1 ), by = 2)]), omega[ , (2 * m - 3)], 1/m ), 1 - rowSums(omega[ , seq(from = 1, to = 2 * (m - 1) - 1, by = 2)]))
        }else{
          omega[ , (2 * m - 1) : (2 * m)] <- cbind( pmax( 1 - omega[ , 2], omega[,1], 1/m) , 1 - omega[ , 1])    
        }}else{
          if((i + 2) < m){
            for(k in (i + 2) : (m - 1)){
              omega[ ,(2 * k - 1) : (2 * k)] <- cbind( omega[ ,2 * (k - 1) - 1], (1 - sum(omega[ , seq(from = 1, to = 2 * (k - 1) - 1, by = 2)]))/(m - k + 1))
            }
          }
          if(2 < m){
            omega[ ,(2 * m - 1) : (2 * m)] <- cbind( pmax( 1 - sum( omega[ ,seq( from = 2, to = 2 * (m - 1), by = 2)]), omega[ ,(2 * m - 3)], 1/m ), 1 - sum(omega[ , seq(from = 1, to = 2 * (m - 1) - 1, by = 2)]))
          }else{
            omega[ ,(2 * m - 1) : (2 * m)] <- cbind(pmax( 1 - omega[ ,2], omega[,1], 1/m), 1 - omega[ ,1])    
          }
        }
      
    

      remove <- explainVal(omegaB = omega, bou = bouR, am = am, amI = amI, indNb = indNb)     
      if(length(remove) != 0){
        idVal <- idVal[-remove,]
      }
      if(length(idVal) == 0){
        ci <- round(data.frame( lower = c(numeric(m-1), 1/m), upper = 1 / (m - 0 : (m-1)) ), digits = 2)
        attr(ci, "uninfo") <- TRUE
        return(ci)
      }
    }
  }
  
  #### omega m
  idVal <- matrix( idVal, ncol = 2*m -2)
  if(dim(idVal)[1] > 1){
    if(m > 2){
      index1 <- (t( matrix( bou[,1], ncol = dim(idVal)[1], nrow = length(bou[,1])) ) <= 
                    matrix( al[2] + al[1] * (m - 1) - rowSums( idVal[ ,seq( 1, (2 * m - 3), 2)]), 
                            nrow = dim(idVal)[1], ncol = length(bou[ ,1]) ) )
      index2 <- (t( matrix( bou[,2], ncol = dim(idVal)[1], nrow = length(bou[,2])) ) >= 
                    matrix( pmax(al[2] + al[1] * (m - 1) - rowSums(idVal[ ,seq( 2, (2 * m - 2), 2)]), (al[2] - al[1])/m + al[1] ), 
                            nrow = dim(idVal)[1], ncol = length(bou[ ,2]) ) )   
    }else{
      index1 <- (t( matrix( bou[,1], ncol = dim(idVal)[1], nrow = length(bou[,1])) ) <= 
                    matrix( al[2] + al[1] * (m - 1) - idVal[ ,1], 
                            nrow = dim(idVal)[1], ncol = length(bou[ ,1])) )
      index2 <- (t( matrix( bou[,2], ncol = dim(idVal)[1], nrow = length(bou[,2])) ) >= 
                    matrix( pmax(al[2] + al[1] * (m - 1) - idVal[ ,2], (al[2] - al[1])/m + al[1] ), 
                            nrow = dim(idVal)[1], ncol = length(bou[ ,2])) )   
    }
  }else{
    if(m > 2){
      index1 <- (t( matrix( bou[,1], ncol = 1, nrow = length(bou[ ,1])) ) <= 
                    matrix( al[2] + al[1] * (m - 1) - sum(idVal[ ,seq(1, (2 * m - 3), 2)]), 
                            nrow = 1, ncol = length(bou[ ,1]) ) )
      index2 <- (t( matrix( bou[,2], ncol = 1, nrow = length(bou[ ,2])) ) >= 
                    matrix( pmax(al[2] + al[1] * (m - 1) - sum(idVal[ ,seq( 2, (2 * m - 2), 2)]),(al[2] - al[1])/m + al[1] ), 
                            nrow = 1, ncol = length(bou[ ,2]) ) )   
    }else{
      index1 <- (t( matrix( bou[,1], ncol = 1, nrow = length(bou[ ,1])) ) <= 
                    matrix( al[2] + al[1] * (m - 1) - idVal[,1], nrow = 1, ncol = length(bou[,1])) )
      index2 <- (t( matrix( bou[,2], ncol = 1, nrow = length(bou[ ,2])) ) >= 
                    matrix( pmax(al[2] + al[1] * (m - 1) - idVal[,2] , (al[2] - al[1])/m + al[1] ), nrow = 1, ncol = length(bou[,2])) )   
    } 
  }
  index <- lapply( split( index1 + index2, seq( max(1, dim(idVal)[1])) ),  function(x) which(x == 2))
  nex <- mapply( function(x,ind) cbind( pmax(bou[x,1], pmax(al[2] + al[1] * (m - 1) 
                                             - sum(idVal[ind, seq( 2, (2 * m - 2), 2)]), (al[2] - al[1])/m + al[1] )),
                                        pmin(bou[x,2], al[2] + al[1] * (m - 1) 
                                             - sum(idVal[ind, seq( 1, (2 * m - 3), 2)]))), 
                 index, seq_along(index), SIMPLIFY = FALSE)  
  nex <- lapply(nex, maxEl)
  lb <- sapply(nex, function(x) length(as.numeric(unlist(x))) / 2)
  
  if(max(lb) == 0){
    ci <- round(data.frame( lower = c(numeric(m-1), 1/m), upper = 1 / (m - 0 : (m-1)) ), digits = 2)
    attr(ci, "uninfo") <- TRUE
    return(ci)
  }
  
  idVal = matrix( apply(idVal, 2, function(x){rep( x, lb)}), ncol = 2 * m - 2)
  nex <- matrix(do.call("rbind", nex), ncol = 2)
  idVal[, seq(2, 2*m - 2, 2)] <- apply(matrix(idVal[, seq(2, 2*m - 2, 2)], ncol = m-1 ), 2, 
                                       function(x) pmin(x, nex[,2]) ) 
  idVal <- cbind(idVal, nex)
  
  if(dim(idVal)[1] > 1){
    idVal[ ,seq(from = 1, to = 2 * m - 1, by = 2)] <- pmax(idVal[ , seq(from = 1, to = 2 * m - 1, by = 2)], 
                                                           al[2] + (m - 1) * al[1] 
                                                           - rowSums(idVal[,seq( 2, 2 * m,  2)]) 
                                                           + idVal[ ,seq( 2,  2 * m, 2)])
    idVal[ ,seq(from = 2, to = 2 * m, by = 2)] <- pmin(idVal[ ,seq(from = 2, to = 2 * m, by = 2)], 
                                                       al[2] + (m - 1) * al[1] 
                                                       - rowSums(idVal[ ,seq( 1, 2 * m - 1, 2)]) 
                                                       + idVal[ ,seq( 1,2 * m - 1, 2)])
  }else{
    idVal[ ,seq(from = 1, to = 2 * m - 1, by = 2)] <- pmax(idVal[ ,seq(from = 1, to = 2 * m - 1, by = 2)], 
                                                           al[2] + (m - 1) * al[1] 
                                                           - sum(idVal[ ,seq( 2, 2 * m, 2)]) 
                                                           + idVal[ ,seq( 2, 2 * m, 2)])
    idVal[ ,seq(from = 2, to = 2 * m, by = 2)] <- pmin(idVal[ ,seq(from = 2, to = 2 * m, by = 2)], 
                                                       al[2] + (m - 1) * al[1] 
                                                       - sum(idVal[ ,seq( 1, 2 * m - 1, 2)]) 
                                                       + idVal[ ,seq( 1, 2 * m - 1, 2)])  
  }
  
  idVal <- unique(round(idVal, digits = -log(tsh) / log(10)), MARGIN = 1)
  
  omega <- unique(round(omegaCal(idVal, al = al), digits = digits), MARGIN = 1)
  
  remove <- which(apply(omega, 1, function(x) 
    min(c(1 - sum(x[seq(1, 2 * m, 2)]), sum(x[seq(2, 2 * m, 2)]) - 1)) < 0    | 
    min(x[seq(2, 2*m, 2)] - x[seq(1, 2*m, 2)] ) < -0.001     
      ) )

  if(length(remove) > 0){
    omega <- omega[-remove, ]
  }
 
  if(length(omega) == 0){
    ci <- round(data.frame( lower = c(numeric(m-1), 1/m), upper = 1 / (m - 0 : (m-1)) ), digits = 2)
    attr(ci, "uninfo") <- TRUE
    return(ci)
  }

  omega <- matrix(omega, ncol = 2 * m)
  omega[, seq(3, 2*m, 2)] <- sapply(seq(3, 2*m, 2), function(x) pmax(omega[,x], omega[, x-2]))
  omega[, seq(2, 2*m -2, 2)] <- sapply(seq(2, 2*m -2, 2), function(x) pmin(omega[,x], omega[, x+2]))

  
  remove <- explainVal(omegaB = omega, bou = bouR, am = am, amI = amI, indNb = indNb)     
  if(length(remove) != 0){
    omega <- omega[-remove,]
  }
  if(length(omega) == 0){
    ci <- round(data.frame( lower = c(numeric(m-1), 1/m), upper = 1 / (m - 0 : (m-1)) ), digits = 2)
    attr(ci, "uninfo") <- TRUE
    return(ci)
  }
  ci <- data.frame(lower = numeric(m), upper = numeric(m))
  if(length(omega) >  2 * m){
    ci$lower <- apply(omega[ ,seq(from = 1, to = 2 * m - 1, by = 2)], 2, min)
    ci$upper <- apply(omega[ ,seq(from = 2, to = 2 * m, by = 2)], 2, max)
  }else{
    ci$lower <- omega[seq(from = 1, to = 2 * m - 1, by = 2)]
    ci$upper <- omega[seq(from = 2, to = 2 * m, by = 2)]
  }
  attr(ci, "uninfo") <- FALSE
  return(ci)
}
