estSources <- function(y, omega, al, beta = 0.01, confband = FALSE, lambda, qB, am){
  invalid <- FALSE
  
  if(missing(y)){
    stop("y must be specified.")
  }
  n <- length(y)
  if(missing(omega)){
    stop("omega must be specified.")
  }
  m <- length(omega)
  if(missing(al)){
    stop("al must be specified.")
  } 
  if( length(al) < 2){
    stop("Alphabet must contain at least two values.")
  }
  al <- sort(al)
  if(missing(lambda)){
    sigma <- stepR::sdrobnorm(y)
    lam <- seq(0,1,0.00001)
    lambda <- max(lam[which(lam / log(1/lam) >= 2 * (sigma * m / (al[2] - al[1]))^2 / n)[1]], 1/n)
  }
  if(!(lambda > 0 & lambda <= 1 )){
    stop("lambda must be between 0 and 1.")
  }
  
  if(missing(am)){
    am <- as.matrix( expand.grid( rep(list(al), m) ) )
  }
  val <- corVal(omega, am = am)

  if(missing(qB)){
    bou <- stepR::bounds(y = y, penalty = "sqrt", alpha = beta)$bounds
  }else{
    bou <- stepR::bounds(y = y, penalty = "sqrt", q = qB)$bounds
  }

  bouM <- .Call('boundsmod', as.numeric(t(bou)), sort(as.numeric(val)) )
  bouM <- cbind( bouM[seq(from = 1, to = length(bouM) - 1, by = 2)], bouM[seq(from = 2, to = length(bouM), by = 2)])
  
  start <- .Call('boundsstart', as.integer(t(bou[ ,1 : 2])))
  
  bou[ ,3 : 4] <- bouM
  
  bou <- list(bounds = bou, start = start, feasible = TRUE)


  if(confband){    
    sigma <- stepR::sdrobnorm(y)
    cN <- (al[2] - al[1]) / m + sqrt(8 * sigma^2 * (1 - log(lambda)) / lambda ) / log(n) 
    
    gEst <- stepR::stepbound(y = y, bounds = bou, confband = TRUE, jumpint = TRUE)
    if(length(gEst$value) > 1){
      gEstCB <- stepR::confband(gEst)
      gEstJI <- cbind( stepR::jumpint(gEst)[-1, 5 : 6], head(stepR::jumpint(gEst)[, 7 : 8], -1))
      jI <- rbind(c(0,0), gEstJI[, 3 : 2])
      
      n <- length(y)
      gEstCB$lower <- gEstCB$lower - cN * log(n) / sqrt(n)
      gEstCB$upper <- gEstCB$upper + cN * log(n) / sqrt(n)
      
      
      cB <- vector("list", length(y))
      for(i in 2 : dim(jI)[1]){
        cB[ ( jI[ i - 1, 2] + 1) : jI[ i, 1] ] <- list( which( gEstCB$lower[jI[ i, 1] ] <= val & gEstCB$upper[ jI[ i, 1] ] >= val ) )
        cB[ (jI[ i, 1] + 1) : jI[ i, 2] ] <- list( unique( c( which( gEstCB$lower[ jI[ i, 1] ] <= val & gEstCB$upper[ jI[ i, 1] ] >= val ), which( gEstCB$lower[ jI[ i, 2] ] <= val & gEstCB$upper[ jI[ i, 2] ] >= val) )))
             
      } 
      if(jI[ dim(jI)[1], 2] < length(y)){
        cB[ (jI[ dim(jI)[1], 2] + 1) : length(y) ] <- list( which( gEstCB$lower[ length(y)] <= val & gEstCB$upper[ length(y) ] >= val ) )
      }
      
      cBA<- list(ind = cB, val = am)
      
      
    }else{
      vcb <- stepR::confband(gEst)[1,]
      cB <- vector("list", 1)
      cB[[1]] <- which(vcb[1] <= val & vcb[2] >= val)
      cB <- rep(cB, length(y))
      cBA<- list(ind = cB, val = am)
    }
        
  }else{
    gEst <- stepR::stepbound(y = y, bounds = bou) 
  }

  if( (max(gEst$value) > 5*max(val))){
    invalid <- TRUE
  }
  
  
  for(i in 1:length(gEst$leftEnd)){
   if(!all((gEst$rightEnd[-(1:i)] - gEst$leftEnd[i])/(lambda*length(y)) 
    > 1 : (length(gEst$leftEnd) - i))){
     invalid <- TRUE
    }
  }
  
  
  fest <- matrix(ncol = m, nrow = length(y)) 
  gest <- numeric(length(y))
  

  for(i in 1 : length(gEst$value)){
    ind <- which.min( ( gEst$value[i] - val )^2 )
    fest[ gEst$leftEnd[i] : gEst$rightEnd[i], ] <- t( matrix( am[ind, ], ncol = gEst$rightEnd[i] - gEst$leftEnd[i] + 1, nrow = m ) )
    gest[ gEst$leftEnd[i] : gEst$rightEnd[i] ] <- rep( val[ind], gEst$rightEnd[i] - gEst$leftEnd[i] + 1 )
  }
  
  
  ans <- list( sources = fest, mixture = gest)
  if(confband){
    attr(ans, "confBand") <- cBA
  }
  attr(ans, "invalid") <- invalid
  return(ans)
}