slamR <- function(y, al, m, lambda, lambdaS, confreg = FALSE, confband = FALSE, alpha = 0.1, beta = 0.01, qA, qB, qAStart = 0){

  if(missing(y)){
    stop("Data must be specified.")
  }
  n <- length(y)
  if( missing(al) ){
    stop("The alphabet must be specified.")
  }
  if( length(al) < 2){
    stop("Alphabet must contain at least two values.")
  }
  al <- sort(al)
  if(missing(m)){
    stop("Number of source functions must be specified.")
  }
  if(m < 2){
    stop("There must be more then one source function.")
  }
  if(missing(lambda)){
    sigma <- stepR::sdrobnorm(y)
    lam <- seq(0,1,0.00001)
    lambda <- max(lam[which(lam / log(1/lam) >= 2 * (sigma * m / (al[2] - al[1]))^2 / n)[1]], 1/n)
  }
  if(missing(lambdaS)){
    lambdaS <- lambda
  }

  
  cr <- confOmega(y = y, al = al, m = m, lambda = lambda, lambdaS = lambdaS, qA = qAStart)
  if(attr(cr, "uninfo")){
    qL <- qAStart
    crL <- cr
    qR <- qAStart
    crR <- cr
    while(attr(crR, "uninfo")){
      qR <- qR + 1
      crR <- confOmega(y = y, al = al, m = m, lambda = lambda, lambdaS = lambdaS, qA = qR)
    }
  }else{
    qR <- qAStart
    crR <- cr
    qL <- qAStart
    crL <- cr
    while(!attr(crL, "uninfo")){
      qL <- qL - 1
      crL <- confOmega(y = y, al = al, m = m, lambda = lambda, lambdaS = lambdaS, qA = qL)
    }
  }
  
  while(qR - qL > 0.1){
    qa <- (qR + qL)/2
    cr <- confOmega(y = y, al = al, m = m, lambda = lambda, lambdaS = lambdaS, qA = qa)
  if(attr(cr, "uninfo")){
    qL <- qa
  }else{
    qR <- qa
    crR <- cr
  }
  }
  
  cr <- crR
  qa <- qR
 
  omega <- round( (cr$lower + cr$upper) / sum(cr), 2)
  
  if(confreg){
    if(missing(qA)){
      ci <- confOmega(y = y, al = al, m = m, lambda = lambda, lambdaS = lambdaS, alpha = alpha)
    }else{
      ci <- confOmega(y = y, al = al, m = m, lambda = lambda, lambdaS = lambdaS, qA = qA)
    }
  }
    
  if(missing(qB)){
    sm <- estSources(y = y, omega = omega, al = al, beta = beta, confband = confband, lambda = lambda)
  }else{
    sm <- estSources(y = y, omega = omega, al = al, confband = confband, qB = qB, lambda = lambda)
  }
  
  ans <- list(omega = omega, sources = sm$sources)
  if(confreg){
    attr(ans, "confReg") <- ci   
  }
  if(confband){
    attr(ans, "confBand") <- attr(sm, "confBand")
  }
  if(missing(qA)){
    attr(ans, "qA") <- qa
  }

  attr(ans, "invalid") <- attr(sm, "invalid")
  
  
  return(ans)
}