
slamSelect <- function(y, al, lambda, delta, q, alpha){
  if(missing(y)){
    stop("Data must be specified.")
  }
  n <- length(y)
  if( missing(al) ){
    stop("The alphabet must be specified.")
  }
  if( length(al) < 2){
    stop("Alphabet must contain at least two values.")
  }
  al <- sort(al)
  if(missing(lambda)){
    sigma <- stepR::sdrobnorm(y)
    lam <- seq(0,1,0.00001)
    lambda <- max(lam[which(lam / log(1/lam) >= 2 * (sigma * m / (al[2] - al[1]))^2 / n)[1]], 1/n)
  }
  
  if(missing(q)){
    if(missing(alpha)){
      q <- sqrt(lambda * delta^4) / 
        (24 * sqrt(2) * max(al)^2 * stepR::sdrobnorm(y))
      
      mE <- estM(y, al, lambda, delta, q)
      if(!is.na(mE)){
        return(mE)
      }
      qL <- q
      qR <- 1
      mER <- estM(y, al, lambda, delta, qR)
      while(is.na(mER) ){
        qR <- qR + 1
        mER <- estM(y, al, lambda, delta, qR)
      }
      while(qR - qL > 0.1){
        q <- (qR + qL)/2
        mE <- estM(y, al, lambda, delta, q)
        if(!is.na(mE) ){
          qR <- q
          mER <- mE
        }else{
          qL <- q
        }
      }
      return(mER)
    }else{
      return(estM(y, al, lambda, delta, alpha = alpha))
    }
  }else{
    return(estM(y, al, lambda, delta, q))
  }
}


estM <- function(y, al, lambda, delta, qA, alpha){
  
  mMax <- floor(min(sqrt(2 / delta - 1/4) - 1/2 , 
              log(max(al) / delta) / log(length(al)), 1/lambda))
  
  if(mMax == 1){
    warning("Lambda and/or delta too large (only a single source function is possible).")
  }
  
  #m = 1
  if(missing(qA)){
    est0 <- estSources(y = y, omega = 1, al = al, beta = alpha, lambda = lambda)
  }else{
    est0 <- estSources(y = y, omega = 1, al = al, qB = qA, lambda = lambda)
  }
  if(!attr(est0, "invalid")){
    return(1)
  } 
  #m= 2
  mE <- 2
  if(missing(qA)){
    oE <- confOmega(y = y, al = al, m = mE, lambda = lambda, alpha = alpha)
  }else{
    oE <- confOmega(y = y, al = al, m = mE, lambda = lambda, qA = qA)
  }
  unvalid <- (attr(oE, "uninfo")|| round(asbCi(oE,al),4) < delta)
  while( unvalid & mE < mMax){
    mE <- mE + 1
    oE <- confOmega(y = y, al = al, m = mE, lambda = lambda, qA = qA)   
    unvalid <- (attr(oE, "uninfo")|| round(asbCi(oE,al),4) < delta)
  }
  
  if(unvalid){
   return(NA)
  }else{
    return(mE)
  }
}


asbCi <- function(oE, al){
  mE <- length(oE$lower)
  if(mE > 7){
    return(asbCiB(oE, al))
  }
  ald <- matrix(al, ncol = length(al), nrow = length(al))
  ald <- unique(sort(as.numeric(ald - t(ald))))
  Ald <- t(expand.grid(rep(list(ald), mE)))
  Ald <- Ald[, -which( colSums(abs(Ald)) == 0)] 
  Ald <- t(Ald)
  
  k = 1
  while(k < dim(Ald)[1]){
    ind <- which(colSums(abs(Ald[k,] + t(Ald))) == 0)
    if(length(ind) > 0){
      Ald <- Ald[- ind, ]
      
    }
    k <- k+1
  }
  
  d <- dim(Ald)[1]
  M <- 2*(max(al) - min(al))
  A <- rbind( cbind(Ald, rep(-1, d), M * diag(d)),
              cbind(- Ald, rep(-1, d), - M * diag(d)),
              cbind(diag(mE) - rbind(rep(0, mE), diag(1, mE-1, mE)), matrix(0,mE,1), matrix(0,mE,d)),
              cbind(rbind(rep(1, mE), rep(-1, mE)), rep(0,2), matrix(0,2,d)),
              cbind(diag(mE), matrix(0,mE,1), matrix(0,mE,d)),
              cbind(-diag(mE), matrix(0,mE,1), matrix(0,mE,d)))    
  b <- c(rep(0,d), rep(-M, d), rep(0,mE), 1, -1, oE$lower, -oE$upper)
  c <- c(rep(0, mE), 1, rep(0,d))
  
 
  # model <- list()
  # model$A          <- A
  # model$obj        <- c
  # model$modelsense <- "max"
  # model$rhs        <- b
  # model$sense      <- rep(">=", dim(A)[1])
  # model$vtype      <- c( rep('C', mE + 1), rep('B', d))
  # params <- list(OutputFlag=0)
  # result <- gurobi::gurobi(model, params)
  # 
  
  result <- lpSolve::lp(direction = "max", objective.in = c, const.mat = A,
                        const.dir = rep(">=", dim(A)[1]), const.rhs = b,
                        binary.vec = (mE+2): (mE + 1 + d))
  
  
  
  result$objval
  
}

asbCiB <- function(oE, al){
  al <- sort(al)
  mE <- length(oE$lower)
  M <- 2*(max(al) - min(al))
  
  Od <- cbind(diag(mE) - rbind(rep(0, mE), diag(1, mE-1, mE)))
  
  A <- rbind( cbind(Od, rep(-1, mE), M * diag(mE)),
              cbind(- Od, rep(-1, mE), - M * diag(mE)),
              cbind(Od, matrix(0,mE,1), matrix(0,mE,mE)),
              cbind(rbind(rep(1, mE), rep(-1, mE)), rep(0,2), matrix(0,2,mE)),
              cbind(diag(mE), matrix(0,mE,1), matrix(0,mE,mE)),
              cbind(-diag(mE), matrix(0,mE,1), matrix(0,mE,mE)))    
  b <- c(rep(0,mE), rep(-M, mE), rep(0,mE), 1, -1, oE$lower, -oE$upper)
  c <- c(rep(0, mE), 1, rep(0,mE))
  
  # 
  # model <- list()
  # model$A          <- A
  # model$obj        <- c
  # model$modelsense <- "max"
  # model$rhs        <- b
  # model$sense      <- rep(">=", dim(A)[1])
  # model$vtype      <- c( rep('C', mE + 1), rep('B', mE))
  # params <- list(OutputFlag=0)
  # result <- gurobi::gurobi(model, params)
  # 
  
  result <- lpSolve::lp(direction = "max", objective.in = c, const.mat = A, 
               const.dir = rep(">=", dim(A)[1]), const.rhs = b,
               binary.vec = (mE+2): (mE + 1 + mE))
 
  
  return(result$objval * (al[2] - al[1]))
}
