\name{confOmega}
\alias{confOmega}
\title{Confidence region for weights in SBSSR models}
\usage{
confOmega(y, al, m, alpha = 0.1, lambda, lambdaS, qA)
}
\arguments{
\item{y}{a vector of numeric observations}

\item{al}{a vector of (at least two) distinct, numeric alphabet values}

\item{m}{an integer bigger than two indicating the number of source functions}

\item{alpha}{significance level in (0,1)}

\item{lambda}{numeric value in (0,1) indicating the minimal distance between any two jumps of the underlying mixture, by default set to detection boundary (see e.g., Chan and Walther ~(2013))}

\item{lambdaS}{numeric value in (0, 1) indicating the relative minimal size of identifiability regions (see \bold{R4} in Behr et al.~(2017)), by default set to \code{lambda} }

\item{qA}{threshold value (if specified \code{alpha} is silently ignored)}
}
\value{

\item{ci}{a data frame with entries \code{lower} and \code{upper} of length \code{m} giving lower and upper bounds for the mixing weights at confidence level \code{1-alpha}}

}
\description{
Calculates an honest confidence region for mixing weights in a Statistical Blind Source Separation Regression (SBSSR) model where the underlying mixture fulfills the separability conditions (see  Behr et al.~(2017)).
}
\examples{
al <- c(0,1)
sources <- rbind(rep(al, each = 100),  rep(al[2:1], each = 100))
omega <- c(0.2, 0.8)
y <- as.numeric(omega \%*\% sources + 0.1 * rnorm(dim(sources)[2]))
ci <- confOmega(y = y, al = al, m = length(omega), alpha = 0.1, lambda = 0.2)
} 
\references{
Behr, M., Munk, A., and Holmes, C. (2017). Multiscale Blind Source Separation. The Annals of Statistics, \emph{arXiv: 1608.07173}. To appear.

Chan, H. P. and Walther, G. (2013). Detection with the scan and the average likelihood ratio. \emph{Statistica Sinica} \bold{23}, 409 - 428 
}

\seealso{
\code{\link{slamR}}, \code{\link{estSources}}, \code{\link{slamSelect}}
}

