\name{estSources}
\alias{estSources}
\title{Source function estimator in SBSSR models}
\usage{
estSources(y, omega, al, beta = 0.01, confband = FALSE, lambda, qB, am)
}
\arguments{
\item{y}{a vector of numeric observations}

\item{omega}{a numeric vector of (estimated) mixing weights}

\item{al}{a vector of (at least two) distinct, numeric alphabet values}

\item{beta}{significance level in (0,1)}

\item{confband}{\code{\link{logical}}, indicates if asymptotically honest multivariate confidence band for source functions should be computed}

\item{lambda}{numeric value in (0,1) indicating the minimal distance between any two jumps of the underlying mixture, by default set to detection boundary (see e.g., Chan and Walther ~(2013))}

\item{qB}{threshold value (if specified \code{beta} is silently ignored)}

\item{am}{matrix of alphabet values}

}
\value{
A list with entries \code{sources} and \code{mixture} and \code{\link{attr}}ibute \code{confBand} (if \code{confband = TRUE}).

\item{sources}{a matrix of dimension \code{c(length(y),length(omega))} of estimated source functions}
\item{mixture}{numeric vector of same length as \code{y} of the estimated mixture}
\item{confBand}{a list with entire \code{ind} and \code{val}, where \code{ind} is itself a list of same length as \code{y} giving column-indices of possible source function values in matrix \code{val} of dimension \code{c(length(al)^m,m)} at confidence level \code{1-beta}}

}
\description{
Estimates source functions in a Statistical Blind Source Separation Regression (SBSSR) model, for given finite alphabet, given number of source functions, and given (or estimated) mixing weights, where the underlying mixture fulfills the separability conditions (see  Behr et al.~(2017)).
}
\examples{
al <- c(0,1)
sources <- rbind(rep(al, each = 100),  rep(al[2:1], each = 100))
omega <- c(0.2, 0.8)
y <- as.numeric(omega \%*\% sources + 0.5 * rnorm(dim(sources)[2]))
est <- estSources(y = y, omega = omega, al = al, confband = TRUE, lambda = 0.2, qB = 2)
confBand <- attr(est, "confBand")
}
\references{
Behr, M., Munk, A., and Holmes, C. (2017). Multiscale Blind Source Separation. The Annals of Statistics, \emph{arXiv: 1608.07173}. To appear.

Chan, H. P. and Walther, G. (2013). Detection with the scan and the average likelihood ratio. \emph{Statistica Sinica} \bold{23}, 409 - 428 
}

\seealso{
\code{\link{slamR}}, \code{\link{confOmega}}, \code{\link{slamSelect}}
}
