\name{slamR}
\alias{slamR}
\title{Demixing in SBSSR models with SLAM}
\usage{
slamR(y, al, m, lambda, lambdaS, confreg = FALSE, confband = FALSE, alpha = 0.1, beta = 0.01, qA, qB, qAStart = 0)
}
\arguments{
\item{y}{a vector of numeric observations}

\item{al}{a vector of (at least two) distinct, numeric alphabet values}

\item{m}{an integer bigger than two indicating the number of source functions}

\item{lambda}{numeric value in (0,1) indicating the minimal distance between any two jumps of the underlying mixture, by default set to detection boundary (see e.g., Chan and Walther ~(2013))}

\item{lambdaS}{numeric value in (0, 1) indicating the relative minimal size of identifiability regions (see \bold{R4} in Behr et al.~(2017)), by default set to \code{lambda} }

\item{confreg}{\code{\link{logical}}, indicates if honest confidence region for mixing weights should be computed}

\item{confband}{\code{\link{logical}}, indicates if asymptotically honest multivariate confidence band for source functions should be computed}

\item{alpha}{significance level for mixing weights in (0,1)}

\item{beta}{significance level for source functions in (0,1)}

\item{qA}{threshold value (if specified \code{alpha} is silently ignored)}

\item{qB}{threshold value (if specified \code{beta} is silently ignored)}

\item{qAStart}{start value for MVT selection criterion}

}
\value{
A list with values \code{omega} and \code{sources} and \code{\link{attr}}ibutes \code{confReg} (if \code{confreg = TRUE}) and \code{confBand} (if \code{confband = TRUE}).

\item{omega}{a numeric vector of length \code{m} of estimated mixing weights}

\item{sources}{a matrix of dimension \code{c(length(y),m)} of estimated source functions}

\item{confReg}{a data frame with entries \code{lower} and \code{upper} of length \code{m} giving lower and upper bounds for the mixing weights at confidence level \code{1-alpha} }

\item{confBand}{a list with entire \code{ind} and \code{val}, where \code{ind} is itself a list of same length as \code{y} giving column-indices of possible source function values in matrix \code{val} of dimension \code{c(length(al)^m,m)} at confidence level \code{1-beta} }
}
\description{
Estimates source functions and mixing weights in a Statistical Blind Source Separation Regression (SBSSR) model, for given finite alphabet and given number of source functions, where the underlying mixture fulfills the separability conditions (see  Behr et al.~(2017)).  
}
\examples{
al <- c(0,1)
sources <- rbind(rep(al, each = 100),  rep(al[2:1], each = 100))
omega <- c(0.35, 0.65)
y <- as.numeric(omega \%*\% sources + 0.3 * rnorm(dim(sources)[2]))
est <- slamR(y = y, al = al, m = length(omega), lambda = 0.2,
               confreg = TRUE, confband = TRUE)
confReg <- attr(est, "confReg")
confBand <- attr(est, "confBand")
}
\references{
Behr, M., Munk, A., and Holmes, C. (2017). Multiscale Blind Source Separation. The Annals of Statistics, \emph{arXiv: 1608.07173}. To appear.


Chan, H. P. and Walther, G. (2013). Detection with the scan and the average likelihood ratio. \emph{Statistica Sinica} \bold{23}, 409 - 428 
}

\seealso{
\code{\link{confOmega}}, \code{\link{estSources}}, \code{\link{slamSelect}}
}

\note{
The threshold parameter for estimating mixing weights is selected according to the MVT-criterion (see  Behr et al.~(2017)).
}
