\name{slamSelect}
\alias{slamSelect}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Model Selection for number of sources in SBSSR model with SLAM selector
}
\description{
Estimates the number of source functions in a Statistical Blind Source Separation Regression (SBSSR) model, for given finite alphabet, where the underlying mixture fulfills the separability conditions (see  Behr et al.~(2017)). 
}
\usage{
slamSelect(y, al, lambda, delta, q, alpha)
}
\arguments{
  \item{y}{a vector of numeric observations}
  \item{al}{a vector of (at least two) distinct, numeric alphabet values}
  \item{lambda}{numeric value in (0,1) indicating the minimal distance between   any two jumps of the underlying mixture, by default set to detection          boundary (see e.g., Chan and Walther ~(2013))}
  \item{delta}{numeric value in (0,1) indicating the minimal alphabet           separation boundary, see  Behr et al.~(2017)}
  \item{alpha}{significance level in (0,1)}
  \item{q}{threshold value (if specified \code{alpha} is silently ignored)}
}

\note{If both, \code{q} and \code{alpha}, are missing, \code{q} is chosen as the optimal asymptotic threshold.}


\value{
A single integer number of estimated number of sources.
}
\references{
Behr, M. (2017). Finite Alphabet Blind Separation. Dissertation (University of Goettingen)

Chan, H. P. and Walther, G. (2013). Detection with the scan and the average likelihood ratio. \emph{Statistica Sinica} \bold{23}, 409 - 428 
}
\examples{
al <- c(0,1)
sources <- rbind(rep(al, each = 100),  rep(al[2:1], each = 100))
omega <- c(0.2, 0.8)
y <- as.numeric(omega \%*\% sources + 0.5 * rnorm(dim(sources)[2]))
slamSelect(y, al, lambda = 0.2, delta = 0.1, q = 1)
slamSelect(y, al, lambda = 0.2, delta = 0.1)
slamSelect(y, al, lambda = 0.2, delta = 0.1, alpha = 0.1)

}

