#include <algorithm>
#include <stdlib.h>
#include <math.h> 
#include <vector>
#include <typeinfo>
#include <R.h>
#include <Rinternals.h>

using namespace std;

//Functors
struct compare_t {
  // Our functor takes a reference to the table and a column to use when comparing rows
  vector <vector <int> > & matrix;
  unsigned  column;
  
  compare_t(
    vector <vector <int> > & matrix,
    unsigned                    column
  ):
    matrix( matrix ),
  column( column )
{ }
// This is the comparison function: taking two rows and comparing them
// by the (previously) specified column
bool operator () ( const vector <int> & lhrow, const vector <int> & rhrow ) const
{
  return lhrow.at( column ) < rhrow.at( column );
}
};

extern "C"{
  
  
  
  SEXP explainv( SEXP bounds, SEXP omega, SEXP aval, SEXP ami, SEXP sizem, SEXP indnb){
    //From R imported variables
    unsigned short rsizem = *INTEGER(sizem);
    double* rbounds = REAL(bounds);
    double* romega = REAL(omega);
    double * raval = REAL(aval);
    int* rami = INTEGER(ami);
    int* rindnb = INTEGER(indnb);
    
    //New variables
    unsigned int sizeb = length(bounds)/4;
    unsigned int sizev = length(aval)/ rsizem;
    unsigned long sizeo = length(omega)/ rsizem /2;
    unsigned int sizeinb = length(indnb);

    
    //Output variables
    SEXP rem;
    PROTECT(rem = allocVector(REALSXP, sizeo));
    double* rrem = REAL(rem); 
    
    for(int i = 0; i< sizeo; i++){
      rrem[i] = 0;
    }
    
    
    
    //Compute output
    for(unsigned long i = 0; i < sizeo; i++){
      
      // compute possible function values (val)
      
      unsigned long dv =  sizev * 2;
      double val[dv];
      
      double oml[rsizem * sizev];
      double omu[rsizem * sizev];
      
      for(int k = 0; k < sizev; k++){
        oml[k*rsizem + rami[k*rsizem]] = romega[i * 2 * rsizem + 2 * rami[k*rsizem] + 1];
        omu[k*rsizem + rami[k*rsizem]] = romega[i * 2 * rsizem + 2 * rami[k*rsizem]];
        
        int lindv = 1;
        int uindv = 1;
        double lvali = romega[i * 2 * rsizem + 2 * rami[k*rsizem] + 1] + romega[i * 2 * rsizem + 2 * rami[k*rsizem + 1] + 1];
        double uvali = romega[i * 2 * rsizem + 2 * rami[k*rsizem]] + romega[i * 2 * rsizem + 2 * rami[k*rsizem + 1]];
        for(int j = 2; j < rsizem; j++){
          lvali = lvali +  romega[i * 2 * rsizem + 2 * rami[k*rsizem + j]];
          uvali = uvali +  romega[i * 2 * rsizem + 2 * rami[k*rsizem + j] + 1];
        }
        
        while(lvali <= 1 && lindv < rsizem-1){
          lindv++;
          lvali = lvali + romega[i * 2 * rsizem + 2 * rami[k*rsizem + lindv] + 1] - romega[i * 2 * rsizem + 2 * rami[k*rsizem + lindv]];
          oml[k*rsizem + rami[k*rsizem + lindv - 1]] = romega[i * 2 * rsizem + 2 * rami[k*rsizem + lindv - 1] + 1];       
        }
        

        
        oml[k*rsizem + rami[k*rsizem + lindv]] = 1 - lvali + romega[i * 2 * rsizem + 2 * rami[k*rsizem + lindv] + 1];
        for(int j = lindv + 1; j < rsizem; j++){
          oml[k*rsizem + rami[k*rsizem + j]] = romega[i * 2 * rsizem + 2 * rami[k*rsizem + j]];       
        }
      
        while(uvali >= 1 && uindv < rsizem-1){
          uindv++;
          uvali = uvali - romega[i * 2 * rsizem + 2 * rami[k*rsizem + uindv] + 1] + romega[i * 2 * rsizem + 2 * rami[k*rsizem + uindv]];
          omu[k*rsizem + rami[k*rsizem + uindv - 1]] = romega[i * 2 * rsizem + 2 * rami[k*rsizem + uindv - 1]];     
        }
        

        
        omu[k*rsizem + rami[k*rsizem + uindv]] = 1 - uvali + romega[i * 2 * rsizem + 2 * rami[k*rsizem + uindv]];
        for(int j = uindv + 1; j < rsizem; j++){
          omu[k*rsizem + rami[k*rsizem + j]] = romega[i * 2 * rsizem + 2 * rami[k*rsizem + j] + 1];       
        }





        val[ k] = 0;
        val[ k + sizev] = 0;
        for(int j =0; j< rsizem; j++){
          val[ k] = val[ k] + raval[k*rsizem + j] * oml[k*rsizem +j];
          val[ k + sizev] = val[k + sizev] + raval[k*rsizem + j] * omu[k*rsizem +j];
        }
      }
      
      

      
      
      // Search bounds which intersect with val
      vector< vector<int> > rboundsv;
      int lengbv = 0;
      
      for(int k = 0; k < sizeb; k++){
        for(int j =0; j < sizev; j++){
          if(rbounds[k*4 + 2] <= val[j + sizev] && val[j] <= rbounds[k*4 + 3]){
            vector<int> newb(2);
            newb[0] = (int)rbounds[k*4];
            newb[1] = 1;
            rboundsv.push_back(newb);
            
            newb[0] = (int)rbounds[k*4 +1];
            newb[1] = -1;
            rboundsv.push_back(newb);
            lengbv++;
            break;
          }
        }	
      }
      
      //Test if rboundsv covers all sampling points
      
      stable_sort( rboundsv.begin(), rboundsv.end(), compare_t( rboundsv, 0 ) );
      
      //Cummulative sum
      int help = 0;
      for(int k = 1; k< lengbv*2-1 ; k++){
        if(help == 1){
          if(rboundsv[k-1][0] +1 < rboundsv[k][0]){
            int helpp = 1;
            for(int l = rboundsv[k-1][0] +1; l < rboundsv[k][0]; l++){
              helpp = 1;
              for( int j = 0; j < sizeinb; j ++){
                if(l == rindnb[j]){
                  helpp = 0;
                  break;
                }
              }

            }
            if(helpp == 1){
              break;
            }
          }
          help = 0;
        }
        rboundsv[k][1] = rboundsv[k][1] + rboundsv[k-1][1];
        if(rboundsv[k][1] == 0){
          help = 1;
        }
      }
      
      
      if(help == 1){
        rrem[i] =1;
      }else{
        rrem[i] = 0;
      }
    }
    
    //Unprotect output
    UNPROTECT(1);
    return(rem);	    
  }
}
